import math, gc, _thread
from machine import Pin, ADC
from rp2 import PIO, StateMachine, asm_pio
from time import sleep_us, sleep_ms


def ValueRange(start: float, end: float, step: float):
    '''Returns a range from a set of floats'''
    while start <= end:
        yield start
        start += step 



def CreateSineValues():
    global Degree_Sample
    global Max_Sine_Value
    
    pwmTable = []
    rvalues = []
    modDepth: float
    sineValue: int
    sineAngle: int
    
    # fill a list of 36 sine curve values per row, starting at smallest range to max 100% values
    # this will be used to determine the inverter output voltage over a range 70 to 100% output
    for modDepth in ValueRange(0.7, 1.0, 0.001):
                
        for sineAngle in range(0, 180, Degree_Sample):
            # create a list of integers mapping a 180 degree half sine curve, 10 bit value with variable %
            sineValue = round(math.sin(sineAngle * math.pi/180) * Max_Sine_Value * modDepth )
            
            # first =  0 value the PIO output still makes a pulsed carrier as the modulation frequency.
            # so ignor it
            if sineValue > 0:
                rvalues.append(sineValue)
            
           
        # add a -1 at the end, this turns off PIO output
        rvalues.append(-1)
        # due to the large array, to prevent out of mem errors, converting each sub list to a tuple also helps
        pwmTable.append(tuple(rvalues))
        print(rvalues)
        print(len(rvalues))
        rvalues = []
        gc.collect()
    
    print('table rows = ', len(pwmTable))
    return pwmTable

def getStateMachine(smId, pin, maxCount, countFreq, prog):
    sm = StateMachine(smId, prog, freq=2 * countFreq, sideset_base=Pin(pin))
    # Use exec() to load max count into ISR
    sm.put(maxCount)
    sm.exec("pull()")
    sm.exec("mov(isr, osr)")
    sm.active(1)
    sm.put(-1) # turn off output
    return sm
    

@asm_pio(sideset_init=PIO.OUT_LOW) # make pin an output and low
def pwmProg():
    pull(noblock) .side(0)      # noblock
    mov(x, osr)          # Pulse width from sine data saved in X, for recycling by noblock
    mov(y, isr)          # ISR must be preloaded with PWM count max
    label("pwmloop")
    jmp(x_not_y, "skip")
    nop()         .side(1)
    label("skip")
    jmp(y_dec, "pwmloop")
    


    
def setCPUFreq(frequency):
    '''Set frequency in hz'''
    if machine.freq != frequency:
        machine.freq(frequency)
        



def getNewIndex():
    ''' running in context of 2nd cpu on seperate thread.
        this loop will be for reading battery voltage and converting to table index as feedback'''
    global lock
    global rowIndex
    global i
    
    while True:
        for i in range(0, 301):
            lock.acquire()
            rowIndex = i
            print(rowIndex, end = ', ')
            lock.release()
            sleep_ms(100)
        









# Main code starts here on CPU 0
#Globals
i = 0
PIO_Frequency = 25_000_000
Degree_Sample = 3
Max_Sine_Value = 1023
PinH_Left = 0
PinH_Right = 1


setCPUFreq(250_000_000)
rowIndex = 30

#get a table of sine value rows (tuples) for 180 degrees spwm modulation
spwmTable = CreateSineValues()

#create 2 state machines
sm0 = getStateMachine(smId=0, pin=PinH_Left, maxCount=Max_Sine_Value, countFreq=PIO_Frequency, prog=pwmProg)
sm1 = getStateMachine(smId=1, pin=PinH_Right, maxCount=Max_Sine_Value, countFreq=PIO_Frequency, prog=pwmProg)

     
spwmValues = ()        
sineValue: int
gc.collect()

lock = _thread.allocate_lock()
_thread.start_new_thread(getNewIndex, ())



while True:
    ''' loops sends individual sine values to PIO to be converted to PWM
        row index set by code running on CPU1 on seperate thread.'''
    
    lock.acquire()
    spwmValues = spwmTable[rowIndex]
    lock.release()
    #update state machines alternately with sine values
    #last value in each row is -1, this turns off PIO output to 0
    #break at this point so the sleep delay doesnt impede the starting of other state machine
    
    for sineValue in spwmValues:
        sm0.put(sineValue)
        if sineValue > 0:
            sleep_us(150)   #36 values per row = 265, 60 values = 150
    
    for sineValue in spwmValues:
        sm1.put(sineValue)
        if sineValue > 0:
            sleep_us(150)
            
    
        
            
            
    


            
          
 




 

    

